@@ -62,56 +62,7 @@
 #'   Multivariate quantile residuals (Kalliovirta and Saikkonen 2010) are also computed and included in the returned object.
 #'   In addition, the returned object contains the estimates and log-likelihood values from all the estimation rounds performed.
 #'   The estimated parameter vector can be obtained at \code{gmvar$params} (and corresponding approximate standard errors
-#'   at \code{gmvar$std_errors}) and it is...
-#'   \describe{
-#'     \item{\strong{For unconstrained models:}}{
-#'       ...a size \eqn{((M(pd^2+d+d(d+1)/2+1)-1)x1)} vector that has form
-#'       \strong{\eqn{\theta}}\eqn{ = }(\strong{\eqn{\upsilon}}\eqn{_{1}},
-#'       ...,\strong{\eqn{\upsilon}}\eqn{_{M}}, \eqn{\alpha_{1},...,\alpha_{M-1}}), where
-#'       \itemize{
-#'         \item \strong{\eqn{\upsilon}}\eqn{_{m}} \eqn{ = (\phi_{m,0},}\strong{\eqn{\phi}}\eqn{_{m}}\eqn{,\sigma_{m})}
-#'         \item \strong{\eqn{\phi}}\eqn{_{m}}\eqn{ = (vec(A_{m,1}),...,vec(A_{m,p})}
-#'         \item and \eqn{\sigma_{m} = vech(\Omega_{m})}, m=1,...,M.
-#'       }
-#'     }
-#'     \item{\strong{For constrained models:}}{
-#'       ...a size \eqn{((M(d+d(d+1)/2+1)+q-1)x1)} vector that has form
-#'       \strong{\eqn{\theta}}\eqn{ = (\phi_{1,0},...,\phi_{M,0},}\strong{\eqn{\psi}}
-#'       \eqn{,\sigma_{1},...,\sigma_{M},\alpha_{1},...,\alpha_{M-1})}, where
-#'       \itemize{
-#'         \item \strong{\eqn{\psi}} \eqn{(qx1)} satisfies (\strong{\eqn{\phi}}\eqn{_{1}}\eqn{,...,}
-#'         \strong{\eqn{\phi}}\eqn{_{M}) =} \strong{\eqn{C \psi}} where \strong{\eqn{C}} is \eqn{(Mpd^2xq)}
-#'         constraint matrix.
-#'       }
-#'     }
-#'     \item{\strong{For structural GMVAR model:}}{
-#'       ...a vector that has the form
-#'       \strong{\eqn{\theta}}\eqn{ = (\phi_{1,0},...,\phi_{M,0},}\strong{\eqn{\phi}}\eqn{_{1},...,}\strong{\eqn{\phi}}\eqn{_{M},
-#'       vec(W),}\strong{\eqn{\lambda}}\eqn{_{2},...,}\strong{\eqn{\lambda}}\eqn{_{M},\alpha_{1},...,\alpha_{M-1})}, where
-#'       \itemize{
-#'         \item\strong{\eqn{\lambda}}\eqn{_{m}=(\lambda_{m1},...,\lambda_{md})} contains the eigenvalues of the \eqn{m}th mixture component.
-#'       }
-#'       \describe{
-#'         \item{\strong{If AR parameters are constrained: }}{Replace \strong{\eqn{\phi}}\eqn{_{1}}\eqn{,...,}
-#'         \strong{\eqn{\phi}}\eqn{_{M}} with \strong{\eqn{\psi}} \eqn{(qx1)} that satisfies (\strong{\eqn{\phi}}\eqn{_{1}}\eqn{,...,}
-#'         \strong{\eqn{\phi}}\eqn{_{M}) =} \strong{\eqn{C \psi}}, as above.}
-#'         \item{\strong{If \eqn{W} is constrained:}}{Remove the zeros from \eqn{vec(W)} and make sure the other entries satisfy
-#'          the sign constraints.}
-#'         \item{\strong{If \eqn{\lambda_{mi}} are constrained:}}{Replace \strong{\eqn{\lambda}}\eqn{_{2},...,}\strong{\eqn{\lambda}}\eqn{_{M}}
-#'          with \strong{\eqn{\gamma}} \eqn{(rx1)} that satisfies (\strong{\eqn{\lambda}}\eqn{_{2}}\eqn{,...,}
-#'         \strong{\eqn{\lambda}}\eqn{_{M}) =} \strong{\eqn{C_{\lambda} \gamma}} where \eqn{C_{\lambda}} is a \eqn{(d(M-1) x r)}
-#'          constraint matrix.}
-#'       }
-#'     }
-#'   }
-#'   Above, \eqn{\phi_{m,0}} is the intercept parameter, \eqn{A_{m,i}} denotes the \eqn{i}th coefficient matrix of the \eqn{m}th
-#'   mixture component, \eqn{\Omega_{m}} denotes the error term covariance matrix of the \eqn{m}:th mixture component, and
-#'   \eqn{\alpha_{m}} is the mixing weight parameter. The \eqn{W} and \eqn{\lambda_{mi}} are structural parameters replacing the
-#'   error term covariance matrices (see Virolainen, 2020). If \eqn{M=1}, \eqn{\alpha_{m}} and \eqn{\lambda_{mi}} are dropped.
-#'   If \code{parametrization=="mean"}, just replace each \eqn{\phi_{m,0}} with regimewise mean \eqn{\mu_{m}}.
-#'   \eqn{vec()} is vectorization operator that stacks columns of a given matrix into a vector. \eqn{vech()} stacks columns
-#'   of a given matrix from the principal diagonal downwards (including elements on the diagonal) into a vector.
-#'   The notation is in line with the cited article by \emph{Kalliovirta, Meitz and Saikkonen (2016)} introducing the GMVAR model.
+#'   at \code{gmvar$std_errors}). See \code{?GMVAR} for the form of the parameter vector, if needed.
 #'
 #'   Remark that the first autocovariance/correlation matrix in \code{$uncond_moments} is for the lag zero,
 #'   the second one for the lag one, etc.
@@ -120,7 +71,7 @@
 #'    \code{predict} and \code{plot}.
 #' @seealso \code{\link{GMVAR}}, \code{\link{iterate_more}}, \code{\link{predict.gmvar}}, \code{\link{profile_logliks}},
 #'   \code{\link{simulateGMVAR}}, \code{\link{quantile_residual_tests}}, \code{\link{print_std_errors}},
-#'   \code{\link{swap_parametrization}}, \code{\link{get_gradient}}, \code{\link{GIRF}}, \code{\link{LR_test}}, \code{\link{Wald_test}},
+#'   \code{\link{swap_parametrization}}, \code{\link{get_gradient}}, \code{\link{GIRF}}, \code{\link{GFEVD}}, \code{\link{LR_test}}, \code{\link{Wald_test}},
 #'   \code{\link{gmvar_to_sgmvar}}, \code{\link{reorder_W_columns}}, \code{\link{swap_W_signs}}, \code{\link{cond_moment_plot}},
 #'   \code{\link{update_numtols}}
 #' @references
@@ -196,10 +147,8 @@
 #' @export
 
 fitGMVAR <- function(data, p, M, conditional=TRUE, parametrization=c("intercept", "mean"), constraints=NULL, same_means=NULL,
-                     structural_pars=NULL, ncalls=floor(10 + 30*log(M)), ncores=min(2, ncalls, parallel::detectCores()),
-                     maxit=500, seeds=NULL, print_res=TRUE, ...) {
+                     structural_pars=NULL, ncalls=floor(10 + 30*log(M)), ncores=2, maxit=500, seeds=NULL, print_res=TRUE, ...) {
 
-  on.exit(closeAllConnections())
   if(!all_pos_ints(c(p, M, ncalls, ncores, maxit))) stop("Arguments p, M, ncalls, ncores, and maxit must be positive integers")
   stopifnot(length(ncalls) == 1)
   if(!is.null(seeds) && length(seeds) != ncalls) stop("The argument 'seeds' should be NULL or a vector of length 'ncalls'")
@@ -227,6 +176,7 @@ fitGMVAR <- function(data, p, M, conditional=TRUE, parametrization=c("intercept"
 
   ### Optimization with the genetic algorithm ###
   cl <- parallel::makeCluster(ncores)
+  on.exit(try(parallel::stopCluster(cl), silent=TRUE)) # Close the cluster on exit, if not already closed.
   parallel::clusterExport(cl, ls(environment(fitGMVAR)), envir = environment(fitGMVAR)) # assign all variables from package:gmvarkit
   parallel::clusterEvalQ(cl, c(library(Brobdingnag), library(mvnfast), library(pbapply)))
 
@@ -287,7 +237,8 @@ fitGMVAR <- function(data, p, M, conditional=TRUE, parametrization=c("intercept"
 
   ### Obtain estimates and standard errors, calculate IC ###
   all_estimates <- lapply(NEWTONresults, function(x) x$par)
-  best_fit <- NEWTONresults[[which(loks == max(loks))[1]]]
+  which_best_fit <- which(loks == max(loks))[1]
+  best_fit <- NEWTONresults[[which_best_fit]]
   params <- best_fit$par
   if(is.null(constraints) && is.null(structural_pars$C_lambda) && is.null(same_means)) {
     params <- sort_components(p=p, M=M, d=d, params=params, structural_pars=structural_pars)
@@ -315,7 +266,8 @@ fitGMVAR <- function(data, p, M, conditional=TRUE, parametrization=c("intercept"
   ret$all_estimates <- all_estimates
   ret$all_logliks <- loks
   ret$which_converged <- converged
-
+  ret$which_round <- which_best_fit # Which estimation round induced the largest log-likelihood?
+  warn_eigens(ret)
   cat("Finished!\n")
   ret
 }
@@ -380,11 +332,23 @@ iterate_more <- function(gmvar, maxit=100, calc_std_errors=TRUE, stat_tol=1e-3,
   res <- optim(par=gmvar$params, fn=fn, gr=gr, method=c("BFGS"), control=list(fnscale=-1, maxit=maxit))
   if(res$convergence == 1) message("The maximum number of iterations was reached! Consired iterating more.")
 
-  GMVAR(data=gmvar$data, p=gmvar$model$p, M=gmvar$model$M, params=res$par,
-        conditional=gmvar$model$conditional, parametrization=gmvar$model$parametrization,
-        constraints=gmvar$model$constraints, same_means=gmvar$model$same_means,
-        structural_pars=gmvar$model$structural_pars, calc_std_errors=calc_std_errors,
-        stat_tol=stat_tol, posdef_tol=posdef_tol)
+  ret <- GMVAR(data=gmvar$data, p=gmvar$model$p, M=gmvar$model$M, params=res$par,
+               conditional=gmvar$model$conditional, parametrization=gmvar$model$parametrization,
+               constraints=gmvar$model$constraints, same_means=gmvar$model$same_means,
+               structural_pars=gmvar$model$structural_pars, calc_std_errors=calc_std_errors,
+               stat_tol=stat_tol, posdef_tol=posdef_tol)
+
+  ret$all_estimates <- gmvar$all_estimates
+  ret$all_logliks <- gmvar$all_logliks
+  ret$which_converged <- gmvar$which_converged
+  if(!is.null(gmvar$which_round)) {
+    ret$which_round <- gmvar$which_round
+    ret$all_estimates[[gmvar$which_round]] <- ret$params
+    ret$all_logliks[gmvar$which_round] <- ret$loglik
+    ret$which_converged[gmvar$which_round] <- res$convergence == 0
+  }
+  warn_eigens(ret)
+  ret
 }
 
 
