@@ -32,6 +32,8 @@ print.gmvar <- function(x, ..., digits=2, summary_print=FALSE) {
   structural_pars <- gmvar$model$structural_pars
   all_mu <- round(get_regime_means(gmvar), digits)
   params <- gmvar$params
+  npars <- length(params)
+  T_obs <- ifelse(is.null(gmvar$data), NA, nrow(gmvar$data))
   params <- reform_constrained_pars(p=p, M=M, d=d, params=params, constraints=constraints,
                                    same_means=same_means, structural_pars=structural_pars)
   if(gmvar$model$parametrization == "mean") {
@@ -45,7 +47,9 @@ print.gmvar <- function(x, ..., digits=2, summary_print=FALSE) {
   all_Omega <- pick_Omegas(p=p, M=M, d=d, params=params, structural_pars=structural_pars)
   alphas <- pick_alphas(p=p, M=M, d=d, params=params)
   cat(ifelse(is.null(structural_pars), "Reduced form", "Structural"), "model:\n")
-  cat(paste0("p = ", p, ", M = ", M, ","),
+  cat(paste0(" p = ", p, ", M = ", M, ", d = ", d, ","),
+      paste0("#parameters = " , npars, ","),
+      ifelse(is.na(T_obs), "\n", paste0("#observations = ", T_obs, " x ", d, ",\n")),
       ifelse(gmvar$model$conditional, "conditional", "exact"),
       "log-likelihood,",
       ifelse(gmvar$model$parametrization == "mean", "mean parametrization,", "intercept parametrization,"),
@@ -59,7 +63,7 @@ print.gmvar <- function(x, ..., digits=2, summary_print=FALSE) {
     all_boldA_eigens <- get_boldA_eigens(gmvar)
     all_omega_eigens <- get_omega_eigens(gmvar)
     form_val2 <- function(txt, val) paste(txt, format_value(val))
-    cat(paste(form_val2("log-likelihood:", gmvar$loglik),
+    cat(paste(form_val2(" log-likelihood:", gmvar$loglik),
                     form_val2("AIC:", IC$AIC),
                     form_val2("HQIC:", IC$HQIC),
                     form_val2("BIC:", IC$BIC),
@@ -67,15 +71,15 @@ print.gmvar <- function(x, ..., digits=2, summary_print=FALSE) {
   }
 
   plus <- c("+", rep(" ", d-1))
-  Y <- paste0("Y", 1:d)
+  Y <- paste0("y", 1:d)
   tmp_names <- paste0("tmp", 1:(p*(d + 2) + d + 2))
 
   for(m in seq_len(M)) {
     count <- 1
     cat(paste("Regime", m), "\n")
     if(summary_print) {
-      cat(paste("Modulus of 'bold A' eigenvalues: ", paste0(format_value(all_boldA_eigens[,m]), collapse=", ")),"\n")
-      cat(paste("Cov. matrix 'Omega' eigenvalues: ", paste0(format_value(all_omega_eigens[,m]), collapse=", ")),"\n")
+      cat(paste("Moduli of 'bold A' eigenvalues: ", paste0(format_value(all_boldA_eigens[,m]), collapse=", ")),"\n")
+      cat(paste("Cov. matrix 'Omega' eigenvalues:", paste0(format_value(all_omega_eigens[,m]), collapse=", ")),"\n")
     }
     cat(paste("Mixing weight:", format_value(alphas[m])), "\n")
     cat("Regime means:", paste0(format_value(all_mu[,m]), collapse=", "), "\n\n")
@@ -297,7 +301,7 @@ print.qrtest <- function(x, ..., digits=3) {
 #' @inheritParams print.gmvarpred
 #' @param x object of class \code{'girf'} generated by the function \code{GIRF}.
 #' @param N_to_print an integer specifying the horizon how far to print the estimates and
-#'   confidence intervals. The default is that all the values are printed (\code{N_to_print=N}).
+#'   confidence intervals. The default is that all the values are printed.
 #' @export
 
 print.girf <- function(x, ..., digits=2, N_to_print) {
@@ -311,15 +315,15 @@ print.girf <- function(x, ..., digits=2, N_to_print) {
     stopifnot(N_to_print %in% 1:nrow(girf_res[[1]]$point_est))
   }
   if(length(girf$which_cumulative) > 0) {
-    cat(paste0("The responses of the variable(s) ",
-               paste0(dimnames(girf$girf_res[[1]]$point_est)[[2]][girf$which_cumulative], collapse=", "),
-               " were accumulated."), "\n\n")
+    cat(paste0("The responses of the variables ",
+               paste0(dimnames(girf_res[[1]]$point_est)[[2]][girf$which_cumulative], collapse=", "),
+               " were cumulated."), "\n\n")
   }
 
   for(i1 in 1:length(girf_res)) {
     if(i1 > 1) cat("------------------------\n")
-    cat(paste0("The GIRF of variable ", i1, ":"), "\n")
-    girf_i1 <- girf$girf_res[[i1]]
+    cat(paste0("The GIRF of shock ", girf$shocks[i1], ":"), "\n")
+    girf_i1 <- girf_res[[i1]]
     for(i2 in 1:dim(girf_i1$conf_ints)[3]) {
       cat(paste0("The response of ", dimnames(girf_i1$conf_ints)[[3]][i2], ":"), "\n")
       df <- as.data.frame(lapply(1:ncol(girf_i1$conf_ints[, , i2]), function(i3) format_value(girf_i1$conf_ints[, i3, i2])))
@@ -336,6 +340,40 @@ print.girf <- function(x, ..., digits=2, N_to_print) {
 }
 
 
+
+#' @describeIn GFEVD print method
+#' @inheritParams print.gmvarpred
+#' @param x object of class \code{'gfevd'} generated by the function \code{GFEVD}.
+#' @param N_to_print an integer specifying the horizon how far to print the estimates.
+#'   The default is that all the values are printed.
+#' @export
+
+print.gfevd <- function(x, ..., digits=2, N_to_print) {
+  gfevd <- x
+  gfevd_res <- gfevd$gfevd_res
+  stopifnot(digits >= 0 & digits%%1 == 0)
+  format_value <- format_valuef(digits)
+  if(missing(N_to_print)) {
+    N_to_print <- nrow(gfevd_res[, , 1]) - 1
+  } else {
+    stopifnot(N_to_print %in% 1:nrow(gfevd_res[, , 1]))
+  }
+  if(length(gfevd$which_cumulative) > 0) {
+    cat(paste0("The responses of the variables ",
+               paste0(dimnames(gfevd_res)[[3]][gfevd$which_cumulative], collapse=", "),
+               " were cumulated."), "\n\n")
+  }
+
+  for(i1 in 1:dim(gfevd_res)[3]) { # Go through GFEVDs of each variable and possibly mixing weights
+    if(i1 > 1) cat("------------------------\n")
+    cat(paste0("The GFEVD for ", dimnames(gfevd_res)[[3]][i1], ":"), "\n")
+    print(round(gfevd_res[1:(N_to_print  + 1), , i1], digits=digits))
+    cat("\n")
+  }
+  invisible(gfevd)
+}
+
+
 #' @describeIn Wald_test print method
 #' @param digits how many significant digits to print?
 #' @param x object of class \code{'wald'} generated by the function \code{Wald_test}.
