@@ -10,16 +10,16 @@
 #'  No argument checks!
 #' @inherit is_stationary references
 
-get_regime_means_int <- function(p, M, d, params, parametrization=c("intercept", "mean"), constraints=NULL, structural_pars=NULL) {
+get_regime_means_int <- function(p, M, d, params, parametrization=c("intercept", "mean"),
+                                 constraints=NULL, same_means=NULL, structural_pars=NULL) {
   parametrization <- match.arg(parametrization)
-  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, constraints=constraints, structural_pars=structural_pars)
+  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, constraints=constraints,
+                                    same_means=same_means, structural_pars=structural_pars)
   structural_pars <- get_unconstrained_structural_pars(structural_pars=structural_pars)
-  if(parametrization == "mean") {
-    return(pick_phi0(p=p, M=M, d=d, params=params, structural_pars=structural_pars))
-  } else {
+  if(parametrization == "intercept") {
     params <- change_parametrization(p=p, M=M, d=d, params=params, constraints=NULL, structural_pars=structural_pars, change_to="mean")
-    return(pick_phi0(p=p, M=M, d=d, params=params, structural_pars=structural_pars))
   }
+  pick_phi0(p=p, M=M, d=d, params=params, structural_pars=structural_pars)
 }
 
 
@@ -70,7 +70,7 @@ get_regime_means <- function(gmvar) {
   check_gmvar(gmvar)
   get_regime_means_int(p=gmvar$model$p, M=gmvar$model$M, d=gmvar$model$d, params=gmvar$params,
                        parametrization=gmvar$model$parametrization, constraints=gmvar$model$constraints,
-                       structural_pars=gmvar$model$structural_pars)
+                       same_means=gmvar$model$same_means, structural_pars=gmvar$model$structural_pars)
 }
 
 
@@ -86,14 +86,14 @@ get_regime_means <- function(gmvar) {
 #'   The subset \code{[, , j, m]} contains the j-1:th lag autocovariance matrix of the m:th regime.
 #' @inherit loglikelihood_int references
 
-get_regime_autocovs_int <- function(p, M, d, params, constraints=NULL, structural_pars=NULL) {
+get_regime_autocovs_int <- function(p, M, d, params, constraints=NULL, same_means=NULL, structural_pars=NULL) {
 
-  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, constraints=constraints, structural_pars=structural_pars)
+  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, constraints=constraints,
+                                    same_means=same_means, structural_pars=structural_pars)
   structural_pars <- get_unconstrained_structural_pars(structural_pars=structural_pars)
   all_A <- pick_allA(p=p, M=M, d=d, params=params, structural_pars=structural_pars)
   all_Omega <- pick_Omegas(p=p, M=M, d=d, params=params, structural_pars=structural_pars)
   all_boldA <- form_boldA(p=p, M=M, d=d, all_A=all_A)
-
   I_dp2 <- diag(nrow=(d*p)^2)
   ZER_lower <- matrix(0, nrow=d*(p-1), ncol=d*p)
   ZER_right <- matrix(0, nrow=d, ncol=d*(p - 1))
@@ -152,7 +152,8 @@ get_regime_autocovs_int <- function(p, M, d, params, constraints=NULL, structura
 get_regime_autocovs <- function(gmvar) {
   check_gmvar(gmvar)
   get_regime_autocovs_int(p=gmvar$model$p, M=gmvar$model$M, d=gmvar$model$d, params=gmvar$params,
-                          constraints=gmvar$model$constraints, structural_pars=gmvar$model$structural_pars)
+                          constraints=gmvar$model$constraints, same_means=gmvar$model$same_means,
+                          structural_pars=gmvar$model$structural_pars)
 }
 
 
@@ -174,12 +175,15 @@ get_regime_autocovs <- function(gmvar) {
 #'   }
 #' @inherit loglikelihood_int references
 
-uncond_moments_int <- function(p, M, d, params, parametrization=c("intercept", "mean"), constraints=NULL, structural_pars=NULL) {
+uncond_moments_int <- function(p, M, d, params, parametrization=c("intercept", "mean"), constraints=NULL,
+                               same_means=NULL, structural_pars=NULL) {
   parametrization <- match.arg(parametrization)
-  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, constraints=constraints, structural_pars=structural_pars) # Remove any constraints
+  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, constraints=constraints, same_means=same_means,
+                                    structural_pars=structural_pars) # Remove any constraints
   structural_pars <- get_unconstrained_structural_pars(structural_pars=structural_pars)
   alphas <- pick_alphas(p=p, M=M, d=d, params=params)
-  reg_means <- get_regime_means_int(p=p, M=M, d=d, params=params, parametrization=parametrization, constraints=NULL, structural_pars=structural_pars)
+  reg_means <- get_regime_means_int(p=p, M=M, d=d, params=params, parametrization=parametrization, constraints=NULL,
+                                    same_means=NULL, structural_pars=structural_pars)
   uncond_mean <- colSums(alphas*t(reg_means))
   tmp <- rowSums(vapply(1:M, function(m) alphas[m]*tcrossprod(reg_means[,m] - uncond_mean), numeric(d*d))) # Vectorized matrix
   reg_autocovs <- get_regime_autocovs_int(p=p, M=M, d=d, params=params, constraints=NULL, structural_pars=structural_pars)
@@ -241,5 +245,160 @@ uncond_moments <- function(gmvar) {
   check_gmvar(gmvar)
   uncond_moments_int(p=gmvar$model$p, M=gmvar$model$M, d=gmvar$model$d, params=gmvar$params,
                      parametrization=gmvar$model$parametrization, constraints=gmvar$model$constraints,
-                     structural_pars=gmvar$model$structural_pars)
+                     same_means=gmvar$model$same_means, structural_pars=gmvar$model$structural_pars)
+}
+
+
+#' @title Calculate the dp-dimensional covariance matrix of p consecutive
+#'  observations of a VAR process
+#'
+#' @description \code{VAR_pcovmat} calculate the dp-dimensional covariance matrix of p consecutive
+#'  observations of a VAR process with the algorithm proposed by McElroy (2017).
+#'
+#' @inheritParams loglikelihood_int
+#' @inheritParams is_stationary
+#' @param all_Am \code{[d, d, p]} array containing the AR coefficient matrices
+#' @param Omega_m the dxd error term covariance matrix
+#' @details
+#'  Most of the code in this function is adapted from the one provided in the
+#'  supplementary material of McElroy (2017). Reproduced under GNU General
+#'  Public License, Copyright (2015) Tucker McElroy.
+#' @return Returns the (dp x dp) covariance matrix.
+#' @references
+#'  \itemize{
+#'    \item McElroy T. 2017. Computation of vector ARMA autocovariances.
+#'          \emph{Statistics and Probability Letters}, \strong{124}, 92-96.
+#'  }
+
+VAR_pcovmat <- function(p, d, all_Am, Omega_m) {
+  # all_Am = all_A[, , , m]
+  # Omega_m = all_Omega[, , m]
+
+  # The K commutation matrix
+  Kcommut <- function(vect) matrix(t(matrix(vect, nrow=d, ncol=d)), ncol=1)
+  Kmat <- apply(diag(d^2), MARGIN=1, FUN=Kcommut)
+
+  # Step 1: vectorized error term covariance matrix for lag zero
+  gamMAvec <- matrix(Omega_m, nrow=d^2, ncol=1)
+
+  # Step 2: error term versus y_t covariance matrix for lag zero
+  Amat <- array(0, dim=c(d^2, p + 1, d^2, 2*p + 1))
+  Arow <-  matrix(nrow=d^2, ncol=d^2*(p + 1))
+  start_inds <- seq(from=ncol(Arow) - d^2 + 1, to=1, by=-d^2)
+  end_inds <- seq(from=ncol(Arow), to=d^2, by=-d^2)
+  Arow[,start_inds[1]:end_inds[1]] <- diag(d^2)
+  for(i1 in 1:p) {
+    Arow[,start_inds[i1 + 1]:end_inds[i1 + 1]] <- -1*diag(d)%x%all_Am[, , i1]
+  }
+  for(i1 in 1:(p + 1)) {
+    Amat[, i1, , i1:(i1 + p)] <- Arow
+  }
+  newA <- array(Amat[, 1:(p + 1), , 1:p], dim=c(d^2, p + 1, d^2, p))
+  for(i1 in 1:(p + 1)) {
+    for(i2 in 1:p) {
+      newA[, i1, , i2] <- newA[, i1, , i2]%*%Kmat
+    }
+  }
+  Amat <- cbind(matrix(Amat[, , , p + 1], nrow=d^2*(p + 1), ncol=d^2),
+                matrix(Amat[, , , (p + 2):(2*p + 1)], nrow=d^2*(p + 1), ncol=d^2*(p)) + matrix(newA[, , , p:1], nrow=d^2*(p + 1), ncol=d^2*p))
+
+  Bmat <- array(0, dim=c(d^2, 1, d^2, p + 1))
+  Brow <-  matrix(nrow=d^2, ncol=d^2*(p + 1))
+  start_inds <- rev(start_inds)
+  end_inds <- rev(end_inds)
+  Brow[,start_inds[1]:end_inds[1]] <- diag(d^2)
+  for(i1 in 1:p) {
+    Brow[,start_inds[i1 + 1]:end_inds[i1 + 1]] <- -1*diag(d)%x%all_Am[, , i1]
+  }
+  Bmat[, 1, , 1:(1 + p)] <- Brow
+  gamMix <- solve(matrix(Bmat[, , , 1], nrow=d^2, ncol=d^2), gamMAvec)
+
+  # Step 3: we pad out Gamma_WX(0) with zeros
+  gamMixTemp <- c(gamMix, rep(0, times=p*d^2))
+
+  # Step 4: compute the Gamma_XX(h) autocovariances for lags h=0,...,p
+  gamVAR <- array(array(solve(Amat, gamMixTemp), dim=c(d, d, p + 1))[, , 1:p], dim=c(d, d, p))
+
+  # After obtaining the autocovariances for lags h=0,...,p-1,
+  # we construct the dp-dimensional covariance matrix for p consecutive
+  # observations of a VAR process.
+
+  # gamVAR contains lag=0 autocovariance in [, , 1], and lag=i in [, , i + 1].
+  # Moreover, we use Gamma_Y(-h) = t(Gamma_Y(h)) and store the transposes
+  # (as taking transpose multiple times uses more computation time):
+  tgamVAR <- array(dim=c(d, d, p))
+  for(i1 in 1:p) {
+    tgamVAR[, , i1] <- t(gamVAR[, , i1])
+  }
+
+  # Finally, we fill in the covariance matrix for p consecutive observations
+  # of the VAR process:
+  Sigma_m <- matrix(nrow=d*p, ncol=d*p)
+  start_inds <- seq(from=1, to=d*(p - 1) + 1, by=d)
+  end_inds <- seq(from=d, to=d*p, by=d)
+  for(i1 in 1:p) { # Go through row blocks
+    for(i2 in 1:p) { # Go through column blocks
+      # If i1 end_ind is larger than i2 end_ind, we consider blocks below block
+      # diagonal (use transpose), and if i1 end_ind is smaller than i2 end_ind,
+      # we consider blocks above block diagonal. If i1 end_ind == i2 end_ind,
+      # we are at the block diagonal.
+      if(end_inds[i1] > end_inds[i2]) { # Below diagonal, use transpose
+        Sigma_m[start_inds[i1]:end_inds[i1], start_inds[i2]:end_inds[i2]] <- tgamVAR[, , abs(i1 - i2) + 1]
+      } else {
+        Sigma_m[start_inds[i1]:end_inds[i1], start_inds[i2]:end_inds[i2]] <- gamVAR[, , abs(i1 - i2) + 1]
+      }
+    }
+  }
+  Sigma_m
+}
+
+
+#' @title Calculate the dp-dimensional covariance matrices \eqn{\Sigma_{m,p}} in the mixing weights
+#'  of the GMVAR model.
+#'
+#' @description \code{get_Sigmas} calculates the dp-dimensional covariance matrices \eqn{\Sigma_{m,p}}
+#'  in the mixing weights of the GMVAR model so that the algorithm proposed by McElroy (2017) employed
+#'  whenever it reduces the computation time.
+#'
+#' @inheritParams is_stationary
+#' @inheritParams form_boldA
+#' @param all_Omega a \code{[d, d, M]} array containing the covariance matrix Omegas
+#' @details
+#'  Calculates the dp-dimensional covariance matrix using the formula (2.1.39) in Lütkepohl (2005) when
+#'  \code{d*p < 12} and using the algorithm proposed by McElroy (2017) otherwise.
+#'
+#'  The code in the implementation of the McElroy's (2017) algorithm (in the function \code{VAR_pcovmat}) is
+#'  adapted from the one provided in the supplementary material of McElroy (2017). Reproduced under GNU General
+#'  Public License, Copyright (2015) Tucker McElroy.
+#' @return Returns a \code{[dp, dp, M]} array containing the dp-dimensional covariance matrices for each regime.
+#' @references
+#'  \itemize{
+#'    \item Kalliovirta L., Meitz M. and Saikkonen P. 2016. Gaussian mixture vector autoregression.
+#'            \emph{Journal of Econometrics}, \strong{192}, 485-498.
+#'    \item Lütkepohl H. 2005. New Introduction to Multiple Time Series Analysis,
+#'            \emph{Springer}.
+#'    \item McElroy T. 2017. Computation of vector ARMA autocovariances.
+#'          \emph{Statistics and Probability Letters}, \strong{124}, 92-96.
+#'  }
+
+get_Sigmas <- function(p, M, d, all_A, all_boldA, all_Omega) {
+  Sigmas <- array(NA, dim=c(d*p, d*p, M)) # Store the (dpxdp) covariance matrices
+  if(d*p < 12) { # d*p < 12
+    # Calculate the covariance matrices Sigma_{m,p} using the equation (2.1.39) in Lütkepohl (2005).
+    I_dp2 <- diag(nrow=(d*p)^2)
+    ZER_lower <- matrix(0, nrow=d*(p - 1), ncol=d*p)
+    ZER_right <- matrix(0, nrow=d, ncol=d*(p - 1))
+    for(m in 1:M) {
+      kronmat <- I_dp2 - kronecker(all_boldA[, , m], all_boldA[, , m])
+      sigma_epsm <- rbind(cbind(all_Omega[, , m], ZER_right), ZER_lower)
+      Sigma_m <- solve(kronmat, vec(sigma_epsm))
+      Sigmas[, , m] <- Sigma_m
+    }
+ } else { # d*p >= 12
+   # Calculate the covariance matrices Sigma_{m,p} using the algorithm proposed my McElroy (2017).
+   for(m in 1:M) {
+     Sigmas[, , m] <- VAR_pcovmat(p=p, d=d, all_Am=all_A[, , , m], Omega_m=all_Omega[, , m])
+   }
+ }
+  Sigmas
 }
